@@ -1,35 +1,37 @@
 from __future__ import annotations
 
 import os
+import tempfile
 from unittest.mock import patch
 
 import numpy as np
 import pandas as pd
 import pytest
+from fs.errors import ResourceNotFound
 
 from napistu import identifiers
-from napistu import sbml_dfs_core
-from napistu.sbml_dfs_core import SBML_dfs
-from napistu.source import Source
-from napistu.ingestion import sbml
-from napistu.modify import pathwayannot
-
 from napistu import identifiers as napistu_identifiers
 from napistu.constants import (
     BQB,
     BQB_DEFINING_ATTRS,
     BQB_DEFINING_ATTRS_LOOSE,
+    CONSENSUS_CHECKS,
     IDENTIFIERS,
     MINI_SBO_FROM_NAME,
+    ONTOLOGIES,
     SBML_DFS,
+    SBML_DFS_SCHEMA,
     SBOTERM_NAMES,
     SCHEMA_DEFS,
-    ONTOLOGIES,
+    SOURCE_SPEC,
 )
+from napistu.ingestion import sbml
 from napistu.ingestion.constants import (
-    INTERACTION_EDGELIST_DEFS,
     INTERACTION_EDGELIST_DEFAULTS,
+    INTERACTION_EDGELIST_DEFS,
 )
+from napistu.sbml_dfs_core import SBML_dfs
+from napistu.source import Source
 
 
 @pytest.fixture
@@ -96,16 +98,9 @@ def test_data():
     return [interaction_edgelist, species_df, compartments_df]
 
 
-def test_drop_cofactors(sbml_dfs):
-    starting_rscs = sbml_dfs.reaction_species.shape[0]
-    reduced_dfs = pathwayannot.drop_cofactors(sbml_dfs)
-
-    assert starting_rscs - reduced_dfs.reaction_species.shape[0] == 20
-
-
-def test_sbml_dfs_from_dict_required(sbml_dfs):
+def test_sbml_dfs_from_dict_required(sbml_dfs, model_source_stub):
     val_dict = {k: getattr(sbml_dfs, k) for k in sbml_dfs._required_entities}
-    sbml_dfs2 = sbml_dfs_core.SBML_dfs(val_dict)
+    sbml_dfs2 = SBML_dfs(val_dict, model_source_stub)
     sbml_dfs2.validate()
 
     for k in sbml_dfs._required_entities:
@@ -280,7 +275,7 @@ def test_sbml_dfs_remove_reactions_check_species(sbml_dfs):
     # find all r_ids for a species and check if
     # removing all these reactions also removes the species
     s_id = sbml_dfs.species.index[0]
-    dat = sbml_dfs.compartmentalized_species.query("s_id == @s_id").merge(
+    dat = sbml_dfs.compartmentalized_species.query(f"{SBML_DFS.S_ID} == @s_id").merge(
         sbml_dfs.reaction_species, left_index=True, right_on=SBML_DFS.SC_ID
     )
     r_ids = dat[SBML_DFS.R_ID].unique()
@@ -289,7 +284,7 @@ def test_sbml_dfs_remove_reactions_check_species(sbml_dfs):
     sbml_dfs.validate()
 
 
-def test_read_sbml_with_invalid_ids():
+def test_read_sbml_with_invalid_ids(model_source_stub):
     SBML_W_BAD_IDS = "R-HSA-166658.sbml"
     test_path = os.path.abspath(os.path.join(__file__, os.pardir))
     sbml_w_bad_ids_path = os.path.join(test_path, "test_data", SBML_W_BAD_IDS)
@@ -297,7 +292,10 @@ def test_read_sbml_with_invalid_ids():
 
     # invalid identifiers still create a valid sbml_dfs
     sbml_w_bad_ids = sbml.SBML(sbml_w_bad_ids_path)
-    assert isinstance(sbml_dfs_core.SBML_dfs(sbml_w_bad_ids), sbml_dfs_core.SBML_dfs)
+    assert isinstance(
+        SBML_dfs(sbml_w_bad_ids, model_source_stub),
+        SBML_dfs,
+    )
 
 
 def test_get_table(sbml_dfs):
@@ -376,7 +374,7 @@ def test_species_status(sbml_dfs):
     )
 
 
-def test_get_identifiers_handles_missing_values():
+def test_get_identifiers_handles_missing_values(model_source_stub):
 
     # Minimal DataFrame with all types
     df = pd.DataFrame(
@@ -432,7 +430,7 @@ def test_get_identifiers_handles_missing_values():
             index=[],
         ),
     }
-    sbml = SBML_dfs(sbml_dict, validate=False)
+    sbml = SBML_dfs(sbml_dict, model_source_stub, validate=False)
     result = sbml.get_identifiers(SBML_DFS.SPECIES)
     assert result.shape[0] == 0 or all(
         result[SBML_DFS.S_ID] == "s1"
@@ -463,110 +461,44 @@ def test_remove_entity_data_success(sbml_dfs_w_data):
     sbml_dfs_w_data.validate()
 
 
-def test_remove_entity_data_nonexistent(sbml_dfs_w_data, caplog):
-    """Test warning when trying to remove nonexistent entity data."""
+def test_remove_entity_data_nonexistent(sbml_dfs_w_data):
+    """Test ValueError when trying to remove nonexistent entity data."""
     # Try to remove nonexistent species data
-    sbml_dfs_w_data._remove_entity_data(SBML_DFS.SPECIES, "nonexistent_label")
-    assert "Label 'nonexistent_label' not found in species_data" in caplog.text
+    with pytest.raises(
+        ValueError, match="Label 'nonexistent_label' not found in species_data"
+    ):
+        sbml_dfs_w_data._remove_entity_data(SBML_DFS.SPECIES, "nonexistent_label")
+
+    # Verify the data is unchanged
     assert set(sbml_dfs_w_data.species_data.keys()) == {"test_species"}
 
-    # Clear the log
-    caplog.clear()
-
     # Try to remove nonexistent reactions data
-    sbml_dfs_w_data._remove_entity_data(SBML_DFS.REACTIONS, "nonexistent_label")
-    assert "Label 'nonexistent_label' not found in reactions_data" in caplog.text
+    with pytest.raises(
+        ValueError, match="Label 'nonexistent_label' not found in reactions_data"
+    ):
+        sbml_dfs_w_data._remove_entity_data(SBML_DFS.REACTIONS, "nonexistent_label")
+
+    # Verify the data is unchanged
     assert set(sbml_dfs_w_data.reactions_data.keys()) == {"test_reactions"}
 
     # Validate the model is still valid
     sbml_dfs_w_data.validate()
 
 
-def test_get_characteristic_species_ids():
+def test_get_characteristic_species_ids(sbml_dfs_characteristic_test_data):
     """
     Test get_characteristic_species_ids function with both dogmatic and non-dogmatic cases.
     """
-    # Create mock species identifiers data
-    mock_species_ids = pd.DataFrame(
-        {
-            SBML_DFS.S_ID: ["s1", "s2", "s3", "s4", "s5"],
-            IDENTIFIERS.IDENTIFIER: [
-                "P12345",
-                "CHEBI:15377",
-                "GO:12345",
-                "P67890",
-                "P67890",
-            ],
-            IDENTIFIERS.ONTOLOGY: ["uniprot", "chebi", "go", "uniprot", "chebi"],
-            IDENTIFIERS.BQB: [
-                BQB.IS,
-                BQB.IS,
-                BQB.HAS_PART,
-                BQB.HAS_VERSION,
-                BQB.ENCODES,
-            ],
-        }
-    )
-
-    # Create minimal required tables for SBML_dfs
-    compartments = pd.DataFrame(
-        {SBML_DFS.C_NAME: ["cytosol"], SBML_DFS.C_IDENTIFIERS: [None]}, index=["C1"]
-    )
-    compartments.index.name = SBML_DFS.C_ID
-    species = pd.DataFrame(
-        {
-            SBML_DFS.S_NAME: ["A"],
-            SBML_DFS.S_IDENTIFIERS: [None],
-            SBML_DFS.S_SOURCE: [None],
-        },
-        index=["s1"],
-    )
-    species.index.name = SBML_DFS.S_ID
-    compartmentalized_species = pd.DataFrame(
-        {
-            SBML_DFS.SC_NAME: ["A [cytosol]"],
-            SBML_DFS.S_ID: ["s1"],
-            SBML_DFS.C_ID: ["C1"],
-            SBML_DFS.SC_SOURCE: [None],
-        },
-        index=["SC1"],
-    )
-    compartmentalized_species.index.name = SBML_DFS.SC_ID
-    reactions = pd.DataFrame(
-        {
-            SBML_DFS.R_NAME: ["rxn1"],
-            SBML_DFS.R_IDENTIFIERS: [None],
-            SBML_DFS.R_SOURCE: [None],
-            SBML_DFS.R_ISREVERSIBLE: [False],
-        },
-        index=["R1"],
-    )
-    reactions.index.name = SBML_DFS.R_ID
-    reaction_species = pd.DataFrame(
-        {
-            SBML_DFS.R_ID: ["R1"],
-            SBML_DFS.SC_ID: ["SC1"],
-            SBML_DFS.STOICHIOMETRY: [1],
-            SBML_DFS.SBO_TERM: ["SBO:0000459"],
-        },
-        index=["RSC1"],
-    )
-    reaction_species.index.name = SBML_DFS.RSC_ID
-
-    sbml_dict = {
-        SBML_DFS.COMPARTMENTS: compartments,
-        SBML_DFS.SPECIES: species,
-        SBML_DFS.COMPARTMENTALIZED_SPECIES: compartmentalized_species,
-        SBML_DFS.REACTIONS: reactions,
-        SBML_DFS.REACTION_SPECIES: reaction_species,
-    }
-    sbml_dfs = SBML_dfs(sbml_dict, validate=False, resolve=False)
+    sbml_dfs = sbml_dfs_characteristic_test_data
+    mock_species_ids = sbml_dfs._mock_species_ids
 
     # Test dogmatic case (default)
     expected_bqbs = BQB_DEFINING_ATTRS + [BQB.HAS_PART]  # noqa: F841
     with patch.object(sbml_dfs, "get_identifiers", return_value=mock_species_ids):
         dogmatic_result = sbml_dfs.get_characteristic_species_ids()
-        expected_dogmatic = mock_species_ids.query("bqb in @expected_bqbs")
+        expected_dogmatic = mock_species_ids.query(
+            f"{IDENTIFIERS.BQB} in @expected_bqbs"
+        )
         pd.testing.assert_frame_equal(
             dogmatic_result, expected_dogmatic, check_like=True
         )
@@ -575,18 +507,20 @@ def test_get_characteristic_species_ids():
     expected_bqbs = BQB_DEFINING_ATTRS_LOOSE + [BQB.HAS_PART]  # noqa: F841
     with patch.object(sbml_dfs, "get_identifiers", return_value=mock_species_ids):
         non_dogmatic_result = sbml_dfs.get_characteristic_species_ids(dogmatic=False)
-        expected_non_dogmatic = mock_species_ids.query("bqb in @expected_bqbs")
+        expected_non_dogmatic = mock_species_ids.query(
+            f"{IDENTIFIERS.BQB} in @expected_bqbs"
+        )
         pd.testing.assert_frame_equal(
             non_dogmatic_result, expected_non_dogmatic, check_like=True
         )
 
 
-def test_sbml_basic_functionality(test_data):
+def test_sbml_basic_functionality(test_data, model_source_stub):
     """Test basic SBML_dfs creation from edgelist."""
     interaction_edgelist, species_df, compartments_df = test_data
 
-    result = sbml_dfs_core.sbml_dfs_from_edgelist(
-        interaction_edgelist, species_df, compartments_df
+    result = SBML_dfs.from_edgelist(
+        interaction_edgelist, species_df, compartments_df, model_source_stub
     )
 
     assert isinstance(result, SBML_dfs)
@@ -599,14 +533,15 @@ def test_sbml_basic_functionality(test_data):
     assert len(result.reaction_species) == 4  # 2 reactions * 2 species each
 
 
-def test_sbml_extra_data_preservation(test_data):
+def test_sbml_extra_data_preservation(test_data, model_source_stub):
     """Test that extra columns are preserved when requested."""
     interaction_edgelist, species_df, compartments_df = test_data
 
-    result = sbml_dfs_core.sbml_dfs_from_edgelist(
+    result = SBML_dfs.from_edgelist(
         interaction_edgelist,
         species_df,
         compartments_df,
+        model_source_stub,
         keep_species_data=True,
         keep_reactions_data="experiment",
     )
@@ -617,12 +552,12 @@ def test_sbml_extra_data_preservation(test_data):
     assert "confidence" in result.reactions_data["experiment"].columns
 
 
-def test_sbml_compartmentalized_naming(test_data):
+def test_sbml_compartmentalized_naming(test_data, model_source_stub):
     """Test compartmentalized species naming convention."""
     interaction_edgelist, species_df, compartments_df = test_data
 
-    result = sbml_dfs_core.sbml_dfs_from_edgelist(
-        interaction_edgelist, species_df, compartments_df
+    result = SBML_dfs.from_edgelist(
+        interaction_edgelist, species_df, compartments_df, model_source_stub
     )
 
     comp_names = result.compartmentalized_species[SBML_DFS.SC_NAME].tolist()
@@ -631,7 +566,7 @@ def test_sbml_compartmentalized_naming(test_data):
     assert "CDKN1A [nucleus]" in comp_names
 
 
-def test_sbml_custom_defaults(test_data):
+def test_sbml_custom_defaults(test_data, model_source_stub):
     """Test custom stoichiometry parameters."""
     interaction_edgelist, species_df, compartments_df = test_data
 
@@ -645,10 +580,11 @@ def test_sbml_custom_defaults(test_data):
         SBOTERM_NAMES.PRODUCT
     )
 
-    result = sbml_dfs_core.sbml_dfs_from_edgelist(
+    result = SBML_dfs.from_edgelist(
         interaction_edgelist,
         species_df,
         compartments_df,
+        model_source_stub,
         interaction_edgelist_defaults=custom_defaults,
     )
 
@@ -690,7 +626,7 @@ def test_validate_table(minimal_valid_sbml_dfs):
                 identifiers.Identifiers([]),
                 identifiers.Identifiers([]),
             ],
-            SBML_DFS.S_SOURCE: [Source(init=True), Source(init=True)],
+            SBML_DFS.S_SOURCE: [Source.empty(), Source.empty()],
         },
         index=pd.Index(["S00001", "S00001"], name=SBML_DFS.S_ID),
     )
@@ -807,3 +743,449 @@ def test_validate_reactions_data(minimal_valid_sbml_dfs):
 def test_validate_passes_with_valid_data(minimal_valid_sbml_dfs):
     """Test that validation passes with completely valid data."""
     minimal_valid_sbml_dfs.validate()  # Should not raise any exceptions
+
+
+@pytest.mark.skip_on_windows
+def test_to_pickle_and_from_pickle(sbml_dfs):
+    """Test saving and loading an SBML_dfs via pickle."""
+
+    # Save to pickle
+    with tempfile.NamedTemporaryFile(suffix=".pkl", delete=False) as tmp_file:
+        pickle_path = tmp_file.name
+
+    try:
+        sbml_dfs.to_pickle(pickle_path)
+
+        # Load from pickle
+        loaded_sbml_dfs = SBML_dfs.from_pickle(pickle_path)
+
+        # Verify the loaded SBML_dfs is identical
+        assert isinstance(loaded_sbml_dfs, SBML_dfs)
+        assert len(loaded_sbml_dfs.compartments) == len(sbml_dfs.compartments)
+        assert len(loaded_sbml_dfs.species) == len(sbml_dfs.species)
+        assert len(loaded_sbml_dfs.reactions) == len(sbml_dfs.reactions)
+        assert len(loaded_sbml_dfs.reaction_species) == len(sbml_dfs.reaction_species)
+
+        # Compare each table, excluding identifier and source columns that contain custom objects
+        for table_name in SBML_DFS_SCHEMA.REQUIRED_ENTITIES:
+            original_df = getattr(sbml_dfs, table_name)
+            loaded_df = getattr(loaded_sbml_dfs, table_name)
+
+            # Get the schema for this table
+            table_schema = SBML_DFS_SCHEMA.SCHEMA[table_name]
+
+            # Create copies to avoid modifying the original DataFrames
+            original_copy = original_df.copy()
+            loaded_copy = loaded_df.copy()
+
+            # Drop identifier and source columns if they exist
+            if SCHEMA_DEFS.ID in table_schema:
+                id_col = table_schema[SCHEMA_DEFS.ID]
+                if id_col in original_copy.columns:
+                    original_copy = original_copy.drop(columns=[id_col])
+                if id_col in loaded_copy.columns:
+                    loaded_copy = loaded_copy.drop(columns=[id_col])
+
+            if SCHEMA_DEFS.SOURCE in table_schema:
+                source_col = table_schema[SCHEMA_DEFS.SOURCE]
+                if source_col in original_copy.columns:
+                    original_copy = original_copy.drop(columns=[source_col])
+                if source_col in loaded_copy.columns:
+                    loaded_copy = loaded_copy.drop(columns=[source_col])
+
+            # Compare the DataFrames without custom object columns
+            pd.testing.assert_frame_equal(original_copy, loaded_copy)
+
+    finally:
+        # Clean up
+        if os.path.exists(pickle_path):
+            os.unlink(pickle_path)
+
+
+@pytest.mark.skip_on_windows
+def test_from_pickle_nonexistent_file():
+    """Test that from_pickle raises appropriate error for nonexistent file."""
+    with tempfile.TemporaryDirectory() as temp_dir:
+        nonexistent_path = os.path.join(temp_dir, "nonexistent_file.pkl")
+        with pytest.raises(ResourceNotFound):
+            SBML_dfs.from_pickle(nonexistent_path)
+
+
+@pytest.mark.skip_on_windows
+def test_pickle_with_species_data(sbml_dfs):
+    """Test pickle functionality with species_data."""
+    # Use the existing sbml_dfs fixture and add species_data
+
+    # Get actual species IDs from the fixture
+    species_ids = sbml_dfs.species.index.tolist()[:2]  # Use first 2 species
+
+    # Add species data
+    species_data = pd.DataFrame(
+        {"expression": [10.5, 20.3], "confidence": [0.8, 0.9]}, index=species_ids
+    )
+    species_data.index.name = SBML_DFS.S_ID
+    sbml_dfs.add_species_data("test_data", species_data)
+
+    # Save to pickle
+    with tempfile.NamedTemporaryFile(suffix=".pkl", delete=False) as tmp_file:
+        pickle_path = tmp_file.name
+
+    try:
+        sbml_dfs.to_pickle(pickle_path)
+
+        # Load from pickle
+        loaded_sbml_dfs = SBML_dfs.from_pickle(pickle_path)
+
+        # Verify species_data is preserved
+        assert "test_data" in loaded_sbml_dfs.species_data
+        pd.testing.assert_frame_equal(
+            loaded_sbml_dfs.species_data["test_data"],
+            sbml_dfs.species_data["test_data"],
+        )
+
+    finally:
+        # Clean up
+        if os.path.exists(pickle_path):
+            os.unlink(pickle_path)
+
+
+def test_get_sources(sbml_dfs_metabolism):
+    """Test get_sources method returns unnest sources table."""
+    # Test with species
+    sources_df = sbml_dfs_metabolism.get_sources(SBML_DFS.SPECIES)
+    assert sources_df is not None
+    assert SOURCE_SPEC.PATHWAY_ID in sources_df.columns
+
+    # Test with reactions
+    sources_df = sbml_dfs_metabolism.get_sources(SBML_DFS.REACTIONS)
+    assert sources_df is not None
+    assert SOURCE_SPEC.PATHWAY_ID in sources_df.columns
+
+    # Test invalid entity type raises error
+    with pytest.raises(
+        ValueError, match="reaction_species does not have a source attribute"
+    ):
+        sbml_dfs_metabolism.get_sources(SBML_DFS.REACTION_SPECIES)
+
+
+def test_get_source_occurrence(sbml_dfs_metabolism):
+    """Test get_source_occurrence method returns source occurrence summary."""
+    # Test with species
+    occurrence_df = sbml_dfs_metabolism.get_source_occurrence(SBML_DFS.SPECIES)
+    assert isinstance(occurrence_df, pd.DataFrame)
+    assert occurrence_df.shape == (129, 4)  # Expected: 129 pathways, 4 columns
+
+    # Test with reactions
+    occurrence_df = sbml_dfs_metabolism.get_source_occurrence(SBML_DFS.REACTIONS)
+    assert isinstance(occurrence_df, pd.DataFrame)
+    assert occurrence_df.shape == (86, 4)  # Expected: 86 pathways, 4 columns
+
+
+def test_get_source_cooccurrence(sbml_dfs_metabolism):
+    """Test get_source_cooccurrence method returns co-occurrence matrix."""
+    # Test with species
+    cooccurrence_df = sbml_dfs_metabolism.get_source_cooccurrence(SBML_DFS.SPECIES)
+    assert isinstance(cooccurrence_df, pd.DataFrame)
+    assert cooccurrence_df.shape == (4, 4)  # Expected: 4x4 square matrix
+
+    # Test with reactions
+    cooccurrence_df = sbml_dfs_metabolism.get_source_cooccurrence(SBML_DFS.REACTIONS)
+    assert isinstance(cooccurrence_df, pd.DataFrame)
+    assert cooccurrence_df.shape == (4, 4)  # Expected: 4x4 square matrix
+
+
+def test_pathway_methods_single_source_error(sbml_dfs):
+    """Test that source methods raise error for single-source models."""
+    # Test get_source_occurrence raises error for single-source model
+    with pytest.raises(ValueError, match="The Source tables for species were empty"):
+        sbml_dfs.get_source_occurrence(SBML_DFS.SPECIES)
+
+    with pytest.raises(ValueError, match="The Source tables for reactions were empty"):
+        sbml_dfs.get_source_occurrence(SBML_DFS.REACTIONS)
+
+    # Test get_pathway_cooccurrence raises error for single-source model
+    with pytest.raises(ValueError, match="The Source tables for species were empty"):
+        sbml_dfs.get_source_cooccurrence(SBML_DFS.SPECIES)
+
+    with pytest.raises(ValueError, match="The Source tables for reactions were empty"):
+        sbml_dfs.get_source_cooccurrence(SBML_DFS.REACTIONS)
+
+
+def test_priority_pathways_filtering(sbml_dfs_metabolism):
+    """Test that priority_pathways parameter correctly filters sources."""
+    # First, get the full co-occurrence matrix to see pathway names
+    full_cooccurrence = sbml_dfs_metabolism.get_source_cooccurrence(SBML_DFS.SPECIES)
+
+    # Select first 2 pathways for testing
+    priority_pathways = list(full_cooccurrence.index)[:2]
+
+    # Test with filtered pathways
+    filtered_cooccurrence = sbml_dfs_metabolism.get_source_cooccurrence(
+        SBML_DFS.SPECIES, priority_pathways=priority_pathways
+    )
+
+    # Verify the filtered result is (2,2)
+    assert filtered_cooccurrence.shape == (2, 2)
+    assert list(filtered_cooccurrence.index) == priority_pathways
+    assert list(filtered_cooccurrence.columns) == priority_pathways
+
+
+def test_get_ontology_occurrence(sbml_dfs_metabolism):
+    """Test get_ontology_occurrence method returns ontology occurrence summary."""
+    # Test with species
+    occurrence_df = sbml_dfs_metabolism.get_ontology_occurrence(SBML_DFS.SPECIES)
+    assert isinstance(occurrence_df, pd.DataFrame)
+    assert occurrence_df.shape == (129, 7)  # Expected: 129 entities, 7 ontologies
+
+    # Test with reactions
+    occurrence_df = sbml_dfs_metabolism.get_ontology_occurrence(SBML_DFS.REACTIONS)
+    assert isinstance(occurrence_df, pd.DataFrame)
+    assert occurrence_df.shape == (86, 4)  # Expected: 86 entities, 4 ontologies
+
+
+def test_get_ontology_cooccurrence(sbml_dfs_metabolism):
+    """Test get_ontology_cooccurrence method returns co-occurrence matrix."""
+    # Test with species
+    cooccurrence_df = sbml_dfs_metabolism.get_ontology_cooccurrence(SBML_DFS.SPECIES)
+    assert isinstance(cooccurrence_df, pd.DataFrame)
+    assert cooccurrence_df.shape == (7, 7)  # Expected: 7x7 square matrix
+
+    # Test with reactions
+    cooccurrence_df = sbml_dfs_metabolism.get_ontology_cooccurrence(SBML_DFS.REACTIONS)
+    assert isinstance(cooccurrence_df, pd.DataFrame)
+    assert cooccurrence_df.shape == (4, 4)  # Expected: 4x4 square matrix
+
+
+def test_get_ontology_cooccurrence_multindex(sbml_dfs_metabolism):
+    """Test get_ontology_cooccurrence method with multi-index columns."""
+    # Test with species and allow_col_multindex=True
+    cooccurrence_df = sbml_dfs_metabolism.get_ontology_cooccurrence(
+        SBML_DFS.SPECIES, allow_col_multindex=True
+    )
+    assert isinstance(cooccurrence_df, pd.DataFrame)
+
+    # Verify it's a square matrix
+    assert cooccurrence_df.shape[0] == cooccurrence_df.shape[1]
+    assert cooccurrence_df.shape == (7, 7)  # Expected: 7x7 square matrix
+
+    # Verify multi-index is present
+    assert isinstance(cooccurrence_df.columns, pd.MultiIndex)
+    assert (
+        len(cooccurrence_df.columns.names) == 2
+    )  # Should have 2 levels (ontology, bqb)
+    assert cooccurrence_df.columns.names == [IDENTIFIERS.ONTOLOGY, IDENTIFIERS.BQB]
+
+    # Verify expected ontologies are present
+    expected_ontologies = {
+        ONTOLOGIES.CHEBI,
+        ONTOLOGIES.PUBMED,
+        ONTOLOGIES.REACTOME,
+        ONTOLOGIES.UNIPROT,
+    }
+    actual_ontologies = set(
+        cooccurrence_df.columns.get_level_values(IDENTIFIERS.ONTOLOGY)
+    )
+    assert expected_ontologies.issubset(actual_ontologies)
+
+    # Test with reactions and allow_col_multindex=True
+    cooccurrence_df = sbml_dfs_metabolism.get_ontology_cooccurrence(
+        SBML_DFS.REACTIONS, allow_col_multindex=True
+    )
+    assert isinstance(cooccurrence_df, pd.DataFrame)
+
+    # Verify it's a square matrix
+    assert cooccurrence_df.shape[0] == cooccurrence_df.shape[1]
+    assert cooccurrence_df.shape == (4, 4)  # Expected: 4x4 square matrix
+
+    # Verify multi-index is present
+    assert isinstance(cooccurrence_df.columns, pd.MultiIndex)
+    assert (
+        len(cooccurrence_df.columns.names) == 2
+    )  # Should have 2 levels (ontology, bqb)
+    assert cooccurrence_df.columns.names == [IDENTIFIERS.ONTOLOGY, IDENTIFIERS.BQB]
+
+    # Verify expected ontologies are present
+    expected_ontologies = {
+        ONTOLOGIES.EC_CODE,
+        ONTOLOGIES.GO,
+        ONTOLOGIES.PUBMED,
+        ONTOLOGIES.REACTOME,
+    }
+    actual_ontologies = set(
+        cooccurrence_df.columns.get_level_values(IDENTIFIERS.ONTOLOGY)
+    )
+    assert expected_ontologies.issubset(actual_ontologies)
+
+
+def test_get_ontology_occurrence_characteristic_only(sbml_dfs_characteristic_test_data):
+    """Test get_ontology_occurrence method with characteristic_only parameter using the new fixture."""
+    sbml_dfs = sbml_dfs_characteristic_test_data
+
+    # Test with species and characteristic_only=True
+    occurrence_df_char = sbml_dfs.get_ontology_occurrence(
+        SBML_DFS.SPECIES, characteristic_only=True
+    )
+
+    assert isinstance(occurrence_df_char, pd.DataFrame)
+    assert occurrence_df_char.shape[0] == 1
+    assert occurrence_df_char.shape[1] == 3
+
+    # Test with species and characteristic_only=False
+    occurrence_df_all = sbml_dfs.get_ontology_occurrence(
+        SBML_DFS.SPECIES, characteristic_only=False
+    )
+
+    assert isinstance(occurrence_df_all, pd.DataFrame)
+    assert occurrence_df_all.shape[0] == 1
+    assert occurrence_df_all.shape[1] == 5
+
+
+def test_get_ontology_x_source_cooccurrence(sbml_dfs_metabolism):
+    """
+    Test get_ontology_x_source_cooccurrence method with metabolism data.
+
+    This test verifies that the method correctly creates a co-occurrence matrix
+    between ontologies and sources using the metabolism fixture.
+    """
+    # Test basic functionality
+    cooccurrence_matrix = sbml_dfs_metabolism.get_ontology_x_source_cooccurrence(
+        SBML_DFS.SPECIES
+    )
+
+    # Verify the result is a DataFrame with correct data types
+    assert isinstance(cooccurrence_matrix, pd.DataFrame)
+    assert (cooccurrence_matrix >= 0).all().all(), "All values should be non-negative"
+    assert cooccurrence_matrix.dtypes.apply(
+        lambda x: pd.api.types.is_integer_dtype(x)
+    ).all(), "All values should be integers"
+
+    # Test with characteristic_only=True
+    char_cooccurrence = sbml_dfs_metabolism.get_ontology_x_source_cooccurrence(
+        SBML_DFS.SPECIES, characteristic_only=True
+    )
+    assert isinstance(char_cooccurrence, pd.DataFrame)
+
+    # Test with custom priority pathways (use actual pathway names from the data)
+    custom_pathways = cooccurrence_matrix.columns[
+        :2
+    ].tolist()  # Use first 2 pathways from the data
+    custom_cooccurrence = sbml_dfs_metabolism.get_ontology_x_source_cooccurrence(
+        SBML_DFS.SPECIES, priority_pathways=custom_pathways
+    )
+    assert isinstance(custom_cooccurrence, pd.DataFrame)
+
+    # Test with reactions
+    reaction_cooccurrence = sbml_dfs_metabolism.get_ontology_x_source_cooccurrence(
+        SBML_DFS.REACTIONS
+    )
+    assert isinstance(reaction_cooccurrence, pd.DataFrame)
+
+    # Test exact dimensions
+    assert cooccurrence_matrix.shape == (
+        7,
+        4,
+    ), f"Expected species co-occurrence shape (7, 4), got {cooccurrence_matrix.shape}"
+    assert reaction_cooccurrence.shape == (
+        4,
+        4,
+    ), f"Expected reaction co-occurrence shape (4, 4), got {reaction_cooccurrence.shape}"
+
+
+def test_post_consensus_checks(sbml_dfs_metabolism):
+    """Test the post_consensus_checks method with a consensus model."""
+
+    # Test with default parameters
+    results = sbml_dfs_metabolism.post_consensus_checks()
+
+    # Verify the structure of the results
+    assert isinstance(results, dict), "Results should be a dictionary"
+
+    # Check that we have results for the default entity types
+    expected_entity_types = [SBML_DFS.SPECIES, SBML_DFS.COMPARTMENTS]
+    for entity_type in expected_entity_types:
+        assert entity_type in results, f"Results should contain {entity_type}"
+        assert isinstance(
+            results[entity_type], dict
+        ), f"Results for {entity_type} should be a dictionary"
+
+    # Check that we have results for the default check types
+    expected_check_types = [
+        CONSENSUS_CHECKS.SOURCE_COOCCURRENCE,
+        CONSENSUS_CHECKS.ONTOLOGY_X_SOURCE_COOCCURRENCE,
+    ]
+    for entity_type in expected_entity_types:
+        for check_type in expected_check_types:
+            assert (
+                check_type in results[entity_type]
+            ), f"Results should contain {check_type} for {entity_type}"
+            assert isinstance(
+                results[entity_type][check_type], pd.DataFrame
+            ), f"Result for {entity_type}/{check_type} should be a DataFrame"
+
+    # Test with custom entity types
+    custom_results = sbml_dfs_metabolism.post_consensus_checks(
+        entity_types=[SBML_DFS.SPECIES],
+        check_types=[CONSENSUS_CHECKS.SOURCE_COOCCURRENCE],
+    )
+
+    assert SBML_DFS.SPECIES in custom_results, "Custom results should contain species"
+    assert (
+        SBML_DFS.COMPARTMENTS not in custom_results
+    ), "Custom results should not contain compartments when not requested"
+    assert (
+        CONSENSUS_CHECKS.SOURCE_COOCCURRENCE in custom_results[SBML_DFS.SPECIES]
+    ), "Custom results should contain source_cooccurrence"
+    assert (
+        CONSENSUS_CHECKS.ONTOLOGY_X_SOURCE_COOCCURRENCE
+        not in custom_results[SBML_DFS.SPECIES]
+    ), "Custom results should not contain ontology_x_source_cooccurrence when not requested"
+
+    # Test with invalid check types
+    with pytest.raises(ValueError, match="Invalid check types"):
+        sbml_dfs_metabolism.post_consensus_checks(check_types=["invalid_check_type"])
+
+    # Test exact dimensions based on the output
+    species_source_cooccurrence = results[SBML_DFS.SPECIES][
+        CONSENSUS_CHECKS.SOURCE_COOCCURRENCE
+    ]
+    species_ontology_cooccurrence = results[SBML_DFS.SPECIES][
+        CONSENSUS_CHECKS.ONTOLOGY_X_SOURCE_COOCCURRENCE
+    ]
+    compartments_source_cooccurrence = results[SBML_DFS.COMPARTMENTS][
+        CONSENSUS_CHECKS.SOURCE_COOCCURRENCE
+    ]
+    compartments_ontology_cooccurrence = results[SBML_DFS.COMPARTMENTS][
+        CONSENSUS_CHECKS.ONTOLOGY_X_SOURCE_COOCCURRENCE
+    ]
+
+    # Test exact dimensions
+    assert species_source_cooccurrence.shape == (
+        4,
+        4,
+    ), f"Expected species source co-occurrence shape (4, 4), got {species_source_cooccurrence.shape}"
+    assert species_ontology_cooccurrence.shape == (
+        7,
+        4,
+    ), f"Expected species ontology co-occurrence shape (7, 4), got {species_ontology_cooccurrence.shape}"
+    assert compartments_source_cooccurrence.shape == (
+        4,
+        4,
+    ), f"Expected compartments source co-occurrence shape (4, 4), got {compartments_source_cooccurrence.shape}"
+    assert compartments_ontology_cooccurrence.shape == (
+        1,
+        4,
+    ), f"Expected compartments ontology co-occurrence shape (1, 4), got {compartments_ontology_cooccurrence.shape}"
+
+    # Test that all results are numeric and not empty
+    for entity_type in [SBML_DFS.SPECIES, SBML_DFS.COMPARTMENTS]:
+        for check_type in [
+            CONSENSUS_CHECKS.SOURCE_COOCCURRENCE,
+            CONSENSUS_CHECKS.ONTOLOGY_X_SOURCE_COOCCURRENCE,
+        ]:
+            df = results[entity_type][check_type]
+            assert (
+                not df.empty
+            ), f"DataFrame for {entity_type}/{check_type} should not be empty"
+            assert pd.api.types.is_numeric_dtype(
+                df.values
+            ), f"Values for {entity_type}/{check_type} should be numeric"
